/* widgets.h

  Created by SMF aka Antoine Laydier (laydier@usa.net)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _WIDGETS_H_
#define _WIDGETS_H_

/*=============================================================================
  HEADERs
=============================================================================*/
#include <ktablistbox.h>
#include <stdio.h>
#include "tag.h"
#include "layer.h"

/*=============================================================================
  TYPEDEFs
 =============================================================================*/
#define MIN_TE 480

/*=============================================================================
  CLASSes
=============================================================================*/

/**
 * This is the class TagEditor. The class defines the TabDialog that contains
 * the tag editor and the informations
 *
 * @short TabWidget class for Tag Editing
 * @author Antoine Laydier (smf@mail.dotcom.fr)
 * @version 0.4.2
 *
 */
class TagEditor : public QTabDialog
{
  Q_OBJECT;
  
 public:

  enum { PAGE_INFO=0 , 
	 PAGE_TAG   
  };
  enum { MENU_ID_RESET=100 ,
	 MENU_ID_SET 
  };
  
  /**
   * The constructor method for class TagEditor
   *
   * @see TagEditor
   * @param parent Widget of TagEditor
   * @param name Name of the widget
   * @param sfolder Folder page number to open first
   * @param _list listBox associed to TagEditor
   *
   */
  TagEditor(QWidget *parent = 0, const char *name = 0, int sfolder=0,
	    QListBox * _list = 0);


  /**
   * The destrucor method for class TagEditor
   *
   */
  ~TagEditor();
     
  /**
   * This method set the opened page to sfolder (see constructor)
   */
  void raiseStartUpPage     ( void );

  /**
   * This close the current file opened if one
   */
  void closeFile            ( void );
  /**
   * This method add a new item in the listbox
   * 
   * @param fileName Name of the file to enqueue
   */
  int  enqueue              (char * fileName);
  
 private:
  
  QPushButton  *pTag_bReset, 
               *pTag_bSet, 
               *bOpen,
               *bNext,
               *bPrev,
               *bClear;
  QWidget      *pages[2];
  QLabel       *pInfo_fileName,
               *pTag_fileName,
               *pInfo_mode,
               *pInfo_type,
               *pInfo_bitRate,
               *pInfo_sampleFreq,
               *pInfo_length;
  QCheckBox    *pTag_CBtitle,
               *pTag_CBartist,
               *pTag_CBalbum,
               *pTag_CByear,
               *pTag_CBgenre,
               *pTag_CBcomment;
  QGroupBox    *pTag_box,
	       *pInfo_box;
  QComboBox    *pTag_cbGenre;
  QListBox     *fileList;
  QLineEdit    *pTag_leTitle,
               *pTag_leArtist,
               *pTag_leAlbum,
               *pTag_leComment,
               *pTag_leYear;

  Tag        * tag;                  // tag informations about current file
  Layer      * layer;                // layer informations about current file
  FILE       * fd;                   // current file descriptor
  int          startup_page;         // page to start with
  bool         restoreStartupPage;   // should we restore the startup page ?

  /**
   * Handles all the resizing of the main window of Kmp3te
   *
   * @param all if this is false the widgets won't be resized but the
   * widget will be updated
   *
   */
  virtual void           resizeEvent    ( QResizeEvent* );

  /**
   * Set all the widgets at theirs places in the TabDialog
   */
  void                   placeWidgets   ( void );

  /**
   * Read tag informations from file
   */
  void                   pTag_Reset     ( void );

  /**
   * Set tag informations in file
   */
  int                    pTag_Set       ( void );

  /**
   * Read tag informations
   *
   * @param rw Open access to file (read/write)
   */
  void                   readTag        ( bool rw );

  /**
   * Initialize the pannel of the TagDialog
   */
  void                   initPannel     ( void );

  /**
   * Search for mp3 files in a directory and all subdirectories and
   * then enqueue them
   */
  void                   processDir     (char * dir);
public slots:

    /**
     * Open the selected file in the associed listbox
     */
    int  open                   ( void );

    /**
     * Handles tabBar selected event
     */
    void tabBarSelected         ( int );

    /**
     * Handles Open button event
     */
    void bOpenActivated         ( void );

    /**
     * Handles Next button event
     */
    void bNextActivated         ( void );

    /**
     * Handles Prev button event
     */
    void bPrevActivated         ( void );

    /**
     * Handles Clear button event
     */
    void bClearActivated        ( void );

    /**
     * Handles Reset button event
     */
    void pTag_bResetActivated   ( void );

    /**
     * Handles Set button event
     */
    void pTag_bSetActivated     ( void );
};

#endif 
